@@ -25,6 +25,8 @@ static ID s_key;
 static ID s_value;
 #endif
 
+static ID s_call;
+
 void msgpack_packer_static_init()
 {
 #ifdef RUBINIUS
@@ -33,6 +35,8 @@ void msgpack_packer_static_init()
     s_key = rb_intern("key");
     s_value = rb_intern("value");
 #endif
+
+    s_call = rb_intern("call");
 }
 
 void msgpack_packer_static_destroy()
@@ -43,8 +47,6 @@ void msgpack_packer_init(msgpack_packer_t* pk)
     memset(pk, 0, sizeof(msgpack_packer_t));
 
     msgpack_buffer_init(PACKER_BUFFER_(pk));
-
-    pk->io = Qnil;
 }
 
 void msgpack_packer_destroy(msgpack_packer_t* pk)
@@ -54,8 +56,6 @@ void msgpack_packer_destroy(msgpack_packer_t* pk)
 
 void msgpack_packer_mark(msgpack_packer_t* pk)
 {
-    rb_gc_mark(pk->io);
-
     /* See MessagePack_Buffer_wrap */
     /* msgpack_buffer_mark(PACKER_BUFFER_(pk)); */
     rb_gc_mark(pk->buffer_ref);
@@ -65,8 +65,6 @@ void msgpack_packer_reset(msgpack_packer_t* pk)
 {
     msgpack_buffer_clear(PACKER_BUFFER_(pk));
 
-    pk->io = Qnil;
-    pk->io_write_all_method = 0;
     pk->buffer_ref = Qnil;
 }
 
@@ -123,9 +121,37 @@ void msgpack_packer_write_hash_value(msgpack_packer_t* pk, VALUE v)
 #endif
 }
 
-static void _msgpack_packer_write_other_value(msgpack_packer_t* pk, VALUE v)
+void msgpack_packer_write_other_value(msgpack_packer_t* pk, VALUE v)
 {
-    rb_funcall(v, pk->to_msgpack_method, 1, pk->to_msgpack_arg);
+    int ext_type;
+
+    VALUE lookup_class;
+
+    /*
+     * Objects of type Integer (Fixnum, Bignum), Float, Symbol and frozen
+     * String have no singleton class and raise a TypeError when trying to get
+     * it. See implementation of #singleton_class in ruby's source code:
+     * VALUE rb_singleton_class(VALUE obj);
+     *
+     * Since all but symbols are already filtered out when reaching this code
+     * only symbols are checked here.
+     */
+    if (SYMBOL_P(v)) {
+      lookup_class = rb_obj_class(v);
+    } else {
+      lookup_class = rb_singleton_class(v);
+    }
+
+    VALUE proc = msgpack_packer_ext_registry_lookup(&pk->ext_registry, lookup_class,
+      &ext_type);
+
+    if(proc != Qnil) {
+        VALUE payload = rb_funcall(proc, s_call, 1, v);
+        StringValue(payload);
+        msgpack_packer_write_ext(pk, ext_type, payload);
+    } else {
+        rb_funcall(v, pk->to_msgpack_method, 1, pk->to_msgpack_arg);
+    }
 }
 
 void msgpack_packer_write_value(msgpack_packer_t* pk, VALUE v)
@@ -162,7 +188,7 @@ void msgpack_packer_write_value(msgpack_packer_t* pk, VALUE v)
         msgpack_packer_write_float_value(pk, v);
         break;
     default:
-        _msgpack_packer_write_other_value(pk, v);
+        msgpack_packer_write_other_value(pk, v);
     }
 }
 
