@@ -19,6 +19,7 @@
 #define MSGPACK_RUBY_PACKER_H__
 
 #include "buffer.h"
+#include "packer_ext_registry.h"
 
 #ifndef MSGPACK_PACKER_IO_FLUSH_THRESHOLD_TO_WRITE_STRING_BODY
 #define MSGPACK_PACKER_IO_FLUSH_THRESHOLD_TO_WRITE_STRING_BODY (1024)
@@ -30,9 +31,8 @@ typedef struct msgpack_packer_t msgpack_packer_t;
 struct msgpack_packer_t {
     msgpack_buffer_t buffer;
 
-    VALUE io;
-    ID io_write_all_method;
     bool compatibility_mode;
+    bool has_symbol_ext_type;
 
     ID to_msgpack_method;
     VALUE to_msgpack_arg;
@@ -41,6 +41,7 @@ struct msgpack_packer_t {
 
     /* options */
     bool comaptibility_mode;
+    msgpack_packer_ext_registry_t ext_registry;
 };
 
 #define PACKER_BUFFER_(pk) (&(pk)->buffer)
@@ -62,12 +63,6 @@ static inline void msgpack_packer_set_to_msgpack_method(msgpack_packer_t* pk,
     pk->to_msgpack_arg = to_msgpack_arg;
 }
 
-static inline void msgpack_packer_set_io(msgpack_packer_t* pk, VALUE io, ID io_write_all_method)
-{
-    pk->io = io;
-    pk->io_write_all_method = io_write_all_method;
-}
-
 void msgpack_packer_reset(msgpack_packer_t* pk);
 
 static inline void msgpack_packer_set_compat(msgpack_packer_t* pk, bool enable)
@@ -261,6 +256,18 @@ static inline void msgpack_packer_write_u64(msgpack_packer_t* pk, uint64_t v)
     }
 }
 
+static inline void msgpack_packer_write_float(msgpack_packer_t* pk, float v)
+{
+    msgpack_buffer_ensure_writable(PACKER_BUFFER_(pk), 5);
+    union {
+        float f;
+        uint32_t u32;
+        char mem[4];
+    } castbuf = { v };
+    castbuf.u32 = _msgpack_be_float(castbuf.u32);
+    msgpack_buffer_write_byte_and_data(PACKER_BUFFER_(pk), 0xca, castbuf.mem, 4);
+}
+
 static inline void msgpack_packer_write_double(msgpack_packer_t* pk, double v)
 {
     msgpack_buffer_ensure_writable(PACKER_BUFFER_(pk), 9);
@@ -345,6 +352,50 @@ static inline void msgpack_packer_write_map_header(msgpack_packer_t* pk, unsigne
     }
 }
 
+static inline void msgpack_packer_write_ext(msgpack_packer_t* pk, int ext_type, VALUE payload)
+{
+    unsigned long len = RSTRING_LEN(payload);
+    switch (len) {
+    case 1:
+        msgpack_buffer_ensure_writable(PACKER_BUFFER_(pk), 2);
+        msgpack_buffer_write_2(PACKER_BUFFER_(pk), 0xd4, ext_type);
+        break;
+    case 2:
+        msgpack_buffer_ensure_writable(PACKER_BUFFER_(pk), 2);
+        msgpack_buffer_write_2(PACKER_BUFFER_(pk), 0xd5, ext_type);
+        break;
+    case 4:
+        msgpack_buffer_ensure_writable(PACKER_BUFFER_(pk), 2);
+        msgpack_buffer_write_2(PACKER_BUFFER_(pk), 0xd6, ext_type);
+        break;
+    case 8:
+        msgpack_buffer_ensure_writable(PACKER_BUFFER_(pk), 2);
+        msgpack_buffer_write_2(PACKER_BUFFER_(pk), 0xd7, ext_type);
+        break;
+    case 16:
+        msgpack_buffer_ensure_writable(PACKER_BUFFER_(pk), 2);
+        msgpack_buffer_write_2(PACKER_BUFFER_(pk), 0xd8, ext_type);
+        break;
+    default:
+        if(len < 256) {
+            msgpack_buffer_ensure_writable(PACKER_BUFFER_(pk), 3);
+            msgpack_buffer_write_2(PACKER_BUFFER_(pk), 0xc7, len);
+            msgpack_buffer_write_1(PACKER_BUFFER_(pk), ext_type);
+        } else if(len < 65536) {
+            msgpack_buffer_ensure_writable(PACKER_BUFFER_(pk), 4);
+            uint16_t be = _msgpack_be16(len);
+            msgpack_buffer_write_byte_and_data(PACKER_BUFFER_(pk), 0xc8, (const void*)&be, 2);
+            msgpack_buffer_write_1(PACKER_BUFFER_(pk), ext_type);
+        } else {
+            msgpack_buffer_ensure_writable(PACKER_BUFFER_(pk), 6);
+            uint32_t be = _msgpack_be32(len);
+            msgpack_buffer_write_byte_and_data(PACKER_BUFFER_(pk), 0xc9, (const void*)&be, 4);
+            msgpack_buffer_write_1(PACKER_BUFFER_(pk), ext_type);
+        }
+    }
+    msgpack_buffer_append_string(PACKER_BUFFER_(pk), payload);
+}
+
 #ifdef COMPAT_HAVE_ENCODING
 static inline bool msgpack_packer_is_binary(VALUE v, int encindex)
 {
@@ -398,7 +449,7 @@ static inline void msgpack_packer_write_string_value(msgpack_packer_t* pk, VALUE
 #endif
 }
 
-static inline void msgpack_packer_write_symbol_value(msgpack_packer_t* pk, VALUE v)
+static inline void msgpack_packer_write_symbol_string_value(msgpack_packer_t* pk, VALUE v)
 {
 #ifdef HAVE_RB_SYM2STR
     /* rb_sym2str is added since MRI 2.2.0 */
@@ -412,6 +463,17 @@ static inline void msgpack_packer_write_symbol_value(msgpack_packer_t* pk, VALUE
 #endif
 }
 
+void msgpack_packer_write_other_value(msgpack_packer_t* pk, VALUE v);
+
+static inline void msgpack_packer_write_symbol_value(msgpack_packer_t* pk, VALUE v)
+{
+    if (pk->has_symbol_ext_type) {
+        msgpack_packer_write_other_value(pk, v);
+    } else {
+        msgpack_packer_write_symbol_string_value(pk, v);
+    }
+}
+
 static inline void msgpack_packer_write_fixnum_value(msgpack_packer_t* pk, VALUE v)
 {
 #ifdef JRUBY
