@@ -24,6 +24,10 @@ extern VALUE cMessagePack_Packer;
 
 void MessagePack_Packer_module_init(VALUE mMessagePack);
 
+VALUE MessagePack_Packer_alloc(VALUE klass);
+
+VALUE MessagePack_Packer_initialize(int argc, VALUE* argv, VALUE self);
+
 VALUE MessagePack_pack(int argc, VALUE* argv);
 
 #endif
