@@ -19,6 +19,8 @@
 #include "buffer_class.h"
 #include "packer_class.h"
 #include "unpacker_class.h"
+#include "factory_class.h"
+#include "extension_value_class.h"
 #include "core_ext.h"
 
 void Init_msgpack(void)
@@ -28,6 +30,8 @@ void Init_msgpack(void)
     MessagePack_Buffer_module_init(mMessagePack);
     MessagePack_Packer_module_init(mMessagePack);
     MessagePack_Unpacker_module_init(mMessagePack);
+    MessagePack_Factory_module_init(mMessagePack);
+    MessagePack_ExtensionValue_module_init(mMessagePack);
     MessagePack_core_ext_module_init();
 }
 
