@@ -11,3 +11,8 @@ else
     require "msgpack/msgpack"
   end
 end
+
+require "msgpack/packer"
+require "msgpack/unpacker"
+require "msgpack/factory"
+require "msgpack/symbol"
